------------------------------------------------------------------------------
--  This file is a part of the LPP VHDL IP LIBRARY
--  Copyright (C) 2009 - 2010, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more Cdetails.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                    Author : Paul LEROY
--                    Mail   : paul.leroy@lpp.polytechnique.fr
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.numeric_std.ALL;
USE IEEE.std_logic_1164.ALL;

LIBRARY staging_lpp;
USE staging_lpp.PLE_general_purpose.ALL;
use staging_lpp.PLE_lpp_fft.all;

ENTITY BUTTERFLY_TOP IS
  GENERIC (
    Sample_SZ    : INTEGER := 16);
  PORT (
    rstn : IN STD_LOGIC;
    clk  : IN STD_LOGIC;

    sample_in_val  : IN  STD_LOGIC;
    sample_out_val : OUT STD_LOGIC;

    Are            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Aim            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Bre            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Bim            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    c_in           : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    cps_in         : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    cms_in         : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);

    op1            : OUT STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    op2            : OUT STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    alu_ctrl       : OUT STD_LOGIC_VECTOR( 2 DOWNTO 0 );
    alu_comp       : OUT STD_LOGIC_VECTOR( 1 DOWNTO 0 );

    butterfly_out  : OUT STD_LOGIC_VECTOR ( 2*Sample_SZ-1 DOWNTO 0);
    sel_out        : OUT STD_LOGIC_VECTOR ( 4 DOWNTO 0)
    );
END BUTTERFLY_TOP;

ARCHITECTURE ar_BUTTERFLY_TOP OF BUTTERFLY_TOP IS

SIGNAL sel_op1      : STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Are Aim X Y Z
SIGNAL sel_op2      : STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Bre Bim C_in cps_in cms_in
SIGNAL sel_xyz      : STD_LOGIC_VECTOR( 2 DOWNTO 0 ); -- X Y Z
SIGNAL alu_ctrl_sig : STD_LOGIC_VECTOR( 2 DOWNTO 0 );
SIGNAL alu_comp_sig : STD_LOGIC_VECTOR( 1 DOWNTO 0 );

BEGIN

    alu_ctrl    <= alu_ctrl_sig;
    alu_comp    <= alu_comp_sig;

  BUTTERFLY_DATAFLOW_1 : BUTTERFLY_DATAFLOW
    GENERIC MAP (
        Sample_SZ   => 16)
    PORT MAP (
        rstn        => rstn,
        clk         => clk,

        Are         => Are,
        Aim         => Aim,
        Bre         => Bre,
        Bim         => Bim,
        c_in        => c_in,
        cps_in      => cps_in,
        cms_in      => cms_in,

        op1         => op1,
        op2         => op2,

        out_alu     => butterfly_out,

        sel_op1     => sel_op1,
        sel_op2     => sel_op2,
        sel_xyz     => sel_xyz,
        alu_ctrl    => alu_ctrl_sig,
        alu_comp    => alu_comp_sig
    );
    
  BUTTERFLY_CTRL_1  :  BUTTERFLY_CTRL
    PORT MAP(
        rstn        => rstn,
        clk         => clk,

        sample_in_val   => sample_in_val,
        sample_out_val  => sample_out_val,

        sel_op1     => sel_op1,
        sel_op2     => sel_op2, 
        sel_xyz     => sel_xyz,
        sel_out     => sel_out,
        alu_ctrl    => alu_ctrl_sig,
        alu_comp    => alu_comp_sig
    );

END ar_BUTTERFLY_TOP;